/*

Copyright 2011 Danilo Gemoli <nylus77@libero.it>

This file is part of Snake_v1.0.

Snake_v1.0 is free software: you can redistribute it and/or
modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Snake_v1.0 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Snake_v1.0.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include "area.h"

extern int tras_col;
extern int tras_rig;

Area::Area( int r, int c )
	: righe( r ), colonne( c )
{
	matrice = new int* [ righe ];
	for( int c = 0; c < righe; c++ )
		matrice[ c ] = new int [ colonne ];
}


Area::Area( const Area& area )
{
	this->righe = area.righe;
	this->colonne = area.colonne;

	matrice = new int* [ righe ];
	for( int r = 0; r < righe; r++ )
		matrice[ r ] = new int [ colonne ];

	for( int r = 0; r < righe; r++ )
		for( int c = 0; c < colonne; c++ )
			matrice[ r ][ c ] = area.matrice[ r ][ c ];
}

Area::~Area()
{
	for( int r = 0; r < righe; r++ )
		delete matrice[ r ];
	delete matrice;
}


int Area::get_righe() const
{
	return righe;
}

int Area::get_colonne() const
{
	return colonne;
}

int Area::get_posizione( int riga, int colonna ) const
{
	if( riga >= 0 && riga < righe && colonna >= 0 && colonna < colonne )
		return matrice[ riga ][ colonna ];

	assert( 0 );
}

Area& Area::set_muro( int tipo_muro )
{
	for( int r = 0; r < righe; r++ )
		for( int c = 0; c < colonne; c++ )
		{
			if( r == 0 || r == righe - 1 )
				matrice[ r ][ c ] = tipo_muro;
			else if( c == 0 || c == colonne - 1 )
				matrice[ r ][ c ] = tipo_muro;
			else
				matrice[ r ][ c ] = 0;
		}

	return *this;
}

Area& Area::set_posizione( int riga, int colonna, int valore )
{
	if( riga >= 0 && riga < righe && colonna >= 0 && colonna < colonne )
	{
		matrice[ riga ][ colonna ] = valore;
		return *this;
	}

	assert( 0 );
}

Area& Area::aggiungi_a_posizione( int riga, int colonna, int valore )
{
	if( riga >= 0 && riga < righe && colonna >= 0 && colonna < colonne )
	{
		matrice[ riga ][ colonna ] += valore;
		return *this;
	}

	assert( 0 );
}

void Area::stampa_matrice() const
{
	for( int r = 0; r < righe; r++ )
		for( int c = 0; c < colonne; c++ )
	    	if( matrice[ r ][ c ] == -1 || matrice[ r ][ c ] == -2 )
			{
				attron( COLOR_PAIR( 5 ) );
				mvprintw( r + tras_rig, c + tras_col, "+" );
				attroff( COLOR_PAIR( 5 ) );
			}
}


void Area::debug( int trasla_riga, int trasla_colonna ) const 
{
	for( int r = 0; r < righe; r++ )
		for( int c = 0; c < colonne; c++ )
		{
			if( matrice[ r ][ c ] == -1 )
				mvprintw( trasla_riga + r, trasla_colonna + c, "-" );
			else if( matrice[ r ][ c ] == 4 || matrice[ r ][ c ] == 8 || matrice [ r ][ c ] == 6 || matrice[ r ][ c ] == 2 || matrice[ r ][ c ] == 5 || matrice[ r ][ c ] == 3 )
			{
				attron( COLOR_PAIR( 1 ) );
				mvprintw( trasla_riga + r, trasla_colonna + c, "%d", matrice[ r ][ c ] );
				attroff( COLOR_PAIR( 1 ) );
			}
			else if( matrice[ r ][ c ] == -21 )
			{
				attron( COLOR_PAIR( 3 ) );
				mvprintw( trasla_riga + r, trasla_colonna + c, "-" );
				attroff( COLOR_PAIR( 3 ) );		
			}
			else
			{	
				attron( COLOR_PAIR( 2 ) );
				mvprintw( trasla_riga + r, trasla_colonna + c, "%d", matrice[ r ][ c ] );
				attroff( COLOR_PAIR( 2 ) );
			}
		}
}







