/*

Copyright 2011 Danilo Gemoli <nylus77@libero.it>

This file is part of Snake_v1.0.

Snake_v1.0 is free software: you can redistribute it and/or
modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Snake_v1.0 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Snake_v1.0.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include "movimento.h"
#include "collisioni.h"
#include "menu.h"

extern int tras_rig;
extern int tras_col;

void un_giocatore( Collisioni& collisioni, Area& area, bool muro_solido, bool cibo_diversificato, long int velocita_serpe );
void due_giocatori( Collisioni& collisioni, Area& area, bool muro_solido, bool cibo_diversificato, long int velocita_serpe );

// DIMENSIONE DELL' AREA DI GIOCO
int righe = 30;
int colonne = 75;

int main()
{	
	init_ncurses();

	unsigned giocatori;
	bool uscita = false;
	bool muro_solido = false;
	bool cibo_diversificato = true;
	long int velocita_serpe = 100000;

	Menu menu( uscita, muro_solido, cibo_diversificato, giocatori, velocita_serpe, righe, colonne );
	
	if( !menu.controllo_dimensione_schermo() )
	{
		nodelay( stdscr, false );
		mvprintw( 0,0, "Allargare le dimensioni del terminale" );
		refresh();
		getch();
		endwin();
		exit( 0 );
	}

	menu.crea_menu();
	Area area( righe, colonne );
	Collisioni collisioni;

	do
	{
		giocatori = 0;
		menu.stampa_menu_generale();
		gestione_schermo( colonne, righe );

		if( muro_solido )
			area.set_muro( static_cast< int >( MURO ) );
		else
			area.set_muro( static_cast< int >( TELETRASPORTO_MURO ) );

		area.stampa_matrice();

		if( giocatori == 2 )
		{
			due_giocatori( collisioni, area, muro_solido, cibo_diversificato, velocita_serpe );	
		}
		else if( giocatori == 1 )
		{
			un_giocatore( collisioni, area, muro_solido, cibo_diversificato, velocita_serpe );
		}	

	}
	while( !uscita );

	menu.elimina_menu();
	endwin();
	return 0;
}


void un_giocatore( Collisioni& collisioni, Area& area, bool muro_solido, bool cibo_diversificato, long int velocita_serpe )
{
	Punteggio punteggio;

    Serpente serpe( ( righe / 2 ) + tras_rig, ( colonne / 2 ) + tras_col, 0, 1, 2, area );
	genera_cibo( area, cibo_diversificato );

	Movimento stato_attuale = FERMO;
	bool game_over = false;
	Stato_gioco stato_gioco = NORMALE;

	while( !game_over )
	{	
		movimento( area, serpe, stato_attuale, stato_gioco );

		if( stato_gioco == NORMALE )
		{
			// TIPO DI MURO PRESENTE ALLE PARETI DELL' AREA
			if( muro_solido )
			{
				if( collisioni.collisione_muro( area, serpe ) )
				{
					collisioni.gestisci_muro( area );
					break;
				}
			}
			else
			{
				if( collisioni.collisione_teletrasporto( area, serpe ) )
					collisioni.gestisci_teletrasporto_muro( area, serpe );
			}

			if( collisioni.collisione_cibo( area, serpe ) )
				collisioni.gestisci_cibo( area, serpe, punteggio, cibo_diversificato );

			if( collisioni.collisione_serpente( area, serpe ) )
			{
				collisioni.gestisci_serpente( area );
				break;
			}

			if( stato_attuale != FERMO )
				serpe.cancella_coda( area );

			punteggio.stampa( serpe, 1, 1 );

			// IN CASO DI DEBUG
			//area.debug( tras_rig, tras_col );

			refresh();
			usleep( velocita_serpe );
		}
		else if( stato_gioco == PAUSA )
		{
			stato_attuale = FERMO;
			stato_gioco = NORMALE;
		}
		else if( stato_gioco == ESCI )
		{
			game_over = true;
			getch();
		}
	}

	clear();
}

void due_giocatori( Collisioni& collisioni, Area& area, bool muro_solido, bool cibo_diversificato, long int velocita_serpe )
{
	Punteggio punteggio_1;
	Punteggio punteggio_2;

	Serpente serpe1( ( righe / 2 ) + tras_rig, ( colonne / 2 ) + tras_col, 0, 1, 2, area );
	Serpente serpe2( ( righe / 2 ) + tras_rig + 7, ( colonne / 2 ) + tras_col + 7, 0, 1, 3, area );

	genera_cibo( area, cibo_diversificato );
	genera_cibo( area, cibo_diversificato );

	bool game_over = false;
	Movimento stato_attuale_1 = FERMO;
	Movimento stato_attuale_2 = FERMO;
	Stato_gioco stato_gioco = NORMALE;

	// PER I PUNTEGGI
	attron( COLOR_PAIR( 5 ) );
	mvprintw( tras_rig - 1, tras_col, "Giocatore verde" );
	mvprintw( tras_rig - 2, tras_col, "Giocatore giallo" );
	attroff( COLOR_PAIR( 5 ) );

	while( !game_over )
	{	
		bool salta = movimento_2( area, serpe1, serpe2, stato_attuale_1, stato_attuale_2, stato_gioco );
	
		if( stato_gioco == NORMALE )
		{
			// TIPO DI MURO PRESENTE ALLE PARETI DELL' AREA
			if( muro_solido )
			{
				if( collisioni.collisione_muro( area, serpe1 ) || collisioni.collisione_muro( area, serpe2 ) )
				{
					collisioni.gestisci_muro( area );
					break;
				}
			}
			else
			{
				if( collisioni.collisione_teletrasporto( area, serpe1 ) )
					collisioni.gestisci_teletrasporto_muro( area, serpe1 );
				if( collisioni.collisione_teletrasporto( area, serpe2 ) )
					collisioni.gestisci_teletrasporto_muro( area, serpe2 );
			}

			if( collisioni.collisione_cibo( area, serpe1 ) )
				collisioni.gestisci_cibo( area, serpe1, punteggio_1, cibo_diversificato );
			if( collisioni.collisione_cibo( area, serpe2 ) )
				collisioni.gestisci_cibo( area, serpe2, punteggio_2, cibo_diversificato );

			if( collisioni.collisione_serpente( area, serpe1 ) || collisioni.collisione_serpente( area, serpe2 ) )
			{
				collisioni.gestisci_serpente( area );
				game_over = true;
			}

			if( !salta )
			{
				if( stato_attuale_1 != FERMO )
					serpe1.cancella_coda( area );
				if( stato_attuale_2 != FERMO )
					serpe2.cancella_coda( area );
			}

			punteggio_1.stampa( serpe1, 1, 1 );
			punteggio_2.stampa( serpe2, 2, 2 );

			// IN CASO DI DEBUG
			//area.debug( tras_rig, tras_col );

			refresh();

			if( !salta )
				usleep( velocita_serpe );
		}
		else if( stato_gioco == PAUSA )
		{
			stato_attuale_1 = FERMO;
			stato_attuale_2 = FERMO;
			stato_gioco = NORMALE;
		}
		else if( stato_gioco == ESCI )
		{
			game_over = true;
		}

	}

	clear();
}

