/*

Copyright 2011 Danilo Gemoli <nylus77@libero.it>

This file is part of Snake_v1.0.

Snake_v1.0 is free software: you can redistribute it and/or
modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Snake_v1.0 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Snake_v1.0.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include "menu.h"

extern int tras_rig;
extern int tras_col;

Menu::Menu( bool& usc, bool& muro, bool& cibo, unsigned& gioc, long int& velocita, int& righe, int& colonne )
	: uscita( usc ), muro_solido( muro ), cibo_diversificato( cibo ), giocatori( gioc ), 
	  velocita_serpe( velocita ), righe_area( righe ), colonne_area( colonne ),
	  fin_menu_gen_y( 6 ), fin_menu_gen_x( 25 ), fin_menu_op_y( 6 ), fin_menu_op_x( 25 )
{
}

bool Menu::controllo_dimensione_schermo() const
{
	// LA FINESTRA DEVE ESSERE SUFFICIENTEMENTE GRANDE DA CONTENERE IL MENU E PER GARANTIRE 
	// UN' ESPERIENZA DI GIOCO DECENTE
	
	if( COLS - fin_menu_gen_y < 0 || LINES - fin_menu_gen_x < 5 )
		return false;

	if( COLS - colonne_area < 0 || LINES - righe_area < 5 )
		return false;

	return true;
}

Menu& Menu::crea_menu()
{
	// CON LA CLASSE STRIG SI HANNO PROBLEMI DI VISUALIZZAZIONE NEL MENU
	vector< char* > scelte;
	scelte.push_back( "1 GIOCATORE" );
	scelte.push_back( "GIOCATORE VS GIOCATORE" );
	scelte.push_back( "OPZIONI" );
	scelte.push_back( "ESCI" );

	oggetti_menu_gen = new ITEM* [ 4 + 1 ];

	for( int x = 0; x < 4; ++x )
		oggetti_menu_gen[ x ] = new_item( scelte[ x ], " " );
	oggetti_menu_gen[ 4 ] = NULL;

	menu_gen = new_menu( oggetti_menu_gen );
	finestra_menu_gen = newwin( fin_menu_gen_y, fin_menu_gen_x, ( LINES / 2 ) - ( fin_menu_gen_y / 2 ), ( COLS / 2 ) - ( fin_menu_gen_x / 2 ) );

	set_menu_win( menu_gen, finestra_menu_gen );

	return *this;
}

Menu& Menu::elimina_menu()
{
	for( int x = 0; x < 4 + 1; ++x )
		free_item( oggetti_menu_gen[ x ] );
	free_menu( menu_gen );
	delwin( finestra_menu_gen );
	return *this;
}

Menu& Menu::crea_menu_opzioni()
{
	vector< char* > scelte;
	scelte.push_back( "VELOCITA' SERPE" );
	scelte.push_back( "MURO SOLIDO" );
	scelte.push_back( "DIVERSIFICAZIONE CIBO" );
	scelte.push_back( "TORNA AL MENU PRINCIPALE" );

	oggetti_menu_op = new ITEM* [ 4 + 1 ];

	for( int x = 0; x < 4; x++ )
		oggetti_menu_op[ x ] = new_item( scelte[ x ], " " );
	oggetti_menu_op[ 4 ] = NULL;
	
	menu_op = new_menu( oggetti_menu_op );
	finestra_menu_op = newwin( fin_menu_op_y, fin_menu_op_x, ( LINES / 2 ) - ( fin_menu_op_y / 2 ), ( COLS / 2 ) - ( fin_menu_op_x / 2 ) );

	set_menu_win( menu_op, finestra_menu_op );
	return *this;
}

Menu& Menu::elimina_menu_opzioni()
{
	unpost_menu( menu_op );
	for( int x = 0; x < 4 + 1; ++x )
		free_item( oggetti_menu_op[ x ] );
	free_menu( menu_op );
	delwin( finestra_menu_op );
	return *this;
}

Menu& Menu::stampa_menu_generale() 
{
	post_menu( menu_gen );
	static int posizione = 0;
	int tasto = 1;
	curs_set( 0 );

	bool uscita_menu = false;
	bool PROBLEMA_MENU = false;

	unsigned logo_x = ( COLS / 2 ) - ( fin_menu_gen_x / 2 ) - 5;
	unsigned logo_y = ( LINES / 2 ) - ( fin_menu_gen_y / 2 );

	while( !uscita_menu )
	{
		mvprintw( logo_y - 10, logo_x, "  ###                 #            " );
		mvprintw( logo_y - 9, logo_x,  " #   #                #            " );
		mvprintw( logo_y - 8, logo_x,  " #      # ##    ###   #   #   ###  " );
		mvprintw( logo_y - 7, logo_x,  "  ###   ##  #      #  #  #   #   # " );
		mvprintw( logo_y - 6, logo_x,  "     #  #   #   ####  ###    ##### " );
		mvprintw( logo_y - 5, logo_x,  " #   #  #   #  #   #  #  #   #     " );
		mvprintw( logo_y - 4, logo_x,  "  ###   #   #   ####  #   #   ###  " );


		tasto = getch();
	
		// IL PROBLEMA DEL MENU ANDREBBE IN QUALCHE MODO RISOLTO
		if( PROBLEMA_MENU )
		{
			PROBLEMA_MENU = false;
			tasto = KEY_UP;
		}

		switch( tasto ) 
		{	
			case KEY_UP :
				if( posizione > 0 )	
					--posizione;
				menu_driver( menu_gen, REQ_UP_ITEM );
				break;

			case KEY_DOWN :
				if( posizione < 3 )
					++posizione;
				menu_driver( menu_gen, REQ_DOWN_ITEM );
				break;
		}

		wrefresh( finestra_menu_gen );	

		if( posizione == 0 && tasto == '\n' )
		{
			// UN GIOCATORE 
			giocatori = 1;
			unpost_menu( menu_gen );
			uscita_menu = true;
			clear();
		}
		else if( posizione == 1 && tasto == '\n' )
		{
			// GIOCATORE VS GIOCATORE 
			giocatori = 2;
			unpost_menu( menu_gen );
			uscita_menu = true;
			clear();
		}
		else if( posizione == 2 && tasto == '\n' )
		{
			// SOTTOMENU OPZIONI
			clear();
			this->crea_menu_opzioni();
			this->stampa_menu_opzioni();
			this->elimina_menu_opzioni();
			clear();
			PROBLEMA_MENU = true;
		}
		else if( posizione == 3 && tasto == '\n' )
		{
			// ESCI
			unpost_menu( menu_gen );
			clear();
			uscita_menu = true;
			uscita = true;
		}

		wrefresh( finestra_menu_gen );
		tasto = 1;
	}

	return *this;
}

Menu& Menu::stampa_menu_opzioni() 
{
	post_menu( menu_op );

	int posizione = 0;
	int tasto = 1;
	bool uscita_menu = false;

	int tras_y = ( LINES / 2 ) - ( fin_menu_op_y / 2 );
	int tras_x = ( COLS / 2 ) - ( fin_menu_op_x / 2 );

	mvprintw( tras_y - 2, tras_x, "Tasti direzionali dx/sx per cambiare le opzioni" );
	mvprintw( tras_y, tras_x + 30, "%d", velocita_serpe );
	if( muro_solido )
		mvprintw( tras_y + 1, tras_x + 30, "SI" );
	else
		mvprintw( tras_y + 1, tras_x + 30, "NO" );

	if( cibo_diversificato )
		mvprintw( tras_y + 2, tras_x + 30, "SI" );
	else
		mvprintw( tras_y + 2, tras_x + 30, "NO" );

	while( !uscita_menu )
	{
		tasto = getch();

		switch( tasto ) 
		{	
			case KEY_UP :
				if( posizione > 0 )	
					--posizione;
				menu_driver( menu_op, REQ_UP_ITEM );
				break;

			case KEY_DOWN :
				if( posizione < 3 )
					++posizione;
				menu_driver( menu_op, REQ_DOWN_ITEM );
				break;
		}

		if( posizione == 0 && ( tasto == KEY_LEFT || tasto == KEY_RIGHT ) )
		{
			// VELOCITA' SERPE 
			if( tasto == KEY_RIGHT && velocita_serpe < 900000 )
				velocita_serpe += 10000;
			else if( tasto == KEY_LEFT && velocita_serpe > 10000 )
				velocita_serpe -= 10000;
			
			for( unsigned x = 0; x < 6; ++x )
				mvaddch( tras_y, tras_x + 30 + x, ' ' );

			mvprintw( tras_y, tras_x + 30, "%d", velocita_serpe );
			refresh();
		}
		else if( posizione == 1 && ( tasto == KEY_LEFT || tasto == KEY_RIGHT ) )
		{
			// MURO SOLIDO 
			if( tasto == KEY_RIGHT )
				muro_solido = true;
			else
				muro_solido = false;

			if( muro_solido )
				mvprintw( tras_y + 1, tras_x + 30, "SI" );
			else
				mvprintw( tras_y + 1, tras_x + 30, "NO" );

			refresh();
		}
		else if( posizione == 2 && ( tasto == KEY_LEFT || tasto == KEY_RIGHT ) )
		{
			// DIVERSIFICAZIONE CIBO
			if( tasto == KEY_RIGHT )
				cibo_diversificato = true;
			else
				cibo_diversificato = false;

			if( cibo_diversificato )
				mvprintw( tras_y + 2, tras_x + 30, "SI" );
			else
				mvprintw( tras_y + 2, tras_x + 30, "NO" );

			refresh();
		}
		else if( posizione == 3 && tasto == '\n' )
		{
			// TORNA AL MENU PRINCIPALE
			uscita_menu = true;
			clear();
		}	

		wrefresh( finestra_menu_op );
		tasto = 1;
	}
	

	return *this;
}

