/*

Copyright 2011 Danilo Gemoli <nylus77@libero.it>

This file is part of Snake_v1.0.

Snake_v1.0 is free software: you can redistribute it and/or
modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Snake_v1.0 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Snake_v1.0.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include "movimento.h"

extern int tras_rig;
extern int tras_col;

void movimento( Area& area, Serpente& serpe, Movimento& stato_attuale, Stato_gioco& stato_gioco )
{
	static bool su = false, giu = false, sinistra = false, destra = false;
	static int mov_precedente = 0;

	switch( stato_attuale )
	{
		case SINISTRA :
			sinistra = true;
			break;

		case DESTRA :
			destra = true;
			break;

		case GIU : 
			giu = true;
			break;

		case SU :
			su = true;
			break;

		case FERMO :
			sinistra = destra = giu = su = false;
			break;
	}
	

	int direzione = getch();

	switch( direzione )
	{
		case KEY_LEFT :
			if( ( mov_precedente == 6 || destra == true ) && serpe.get_lunghezza() > 1 )
				break;
			mov_precedente = 0;

			stato_attuale = SINISTRA;
			sinistra = true;
			giu = false, destra = false, su = false;
			break;

		case KEY_RIGHT :
			if( ( mov_precedente == 4 || sinistra == true ) && serpe.get_lunghezza() > 1 )
				break;
			mov_precedente = 0;

			stato_attuale = DESTRA;
			destra = true;
			giu = false, sinistra = false, su = false;
			break;

		case KEY_UP :
			if( ( mov_precedente == 2 || giu == true ) && serpe.get_lunghezza() > 1 )
				break;
			mov_precedente = 0;

			stato_attuale = SU;
			su = true;
			giu = false, sinistra = false, destra = false;
			break;

		case KEY_DOWN :
			if( ( mov_precedente == 8 || su == true ) && serpe.get_lunghezza() > 1 )
				break;
			mov_precedente = 0;

			stato_attuale = GIU;
			giu = true;
			su = false, sinistra = false, destra = false;
			break;

		case 'p' :
			stato_gioco = PAUSA;

			if( su ) mov_precedente = 8;
			else if( sinistra ) mov_precedente = 4;
			else if( destra ) mov_precedente = 6;
			else if( giu ) mov_precedente = 2;

			sinistra = destra = giu = su = false;
			break;

		// TASTO ESC
		case 27 :
			stato_gioco = ESCI;
			break;
	}

	if( su )
		serpe.muovi_su( area );
	else if( giu )
		serpe.muovi_giu( area );
	else if( destra )
		serpe.muovi_destra( area );
	else if( sinistra )
		serpe.muovi_sinistra( area );
}

bool movimento_2( Area& area, Serpente& serpe1, Serpente& serpe2, Movimento& stato_attuale_1, Movimento& stato_attuale_2, Stato_gioco& stato_gioco )
{
	static bool su1 = false, giu1 = false, sinistra1 = false, destra1 = false;
	static bool su2 = false, giu2 = false, sinistra2 = false, destra2 = false;
	static int mov_precedente1 = 0;
	static int mov_precedente2 = 0;

	if( stato_attuale_1 == SINISTRA ) sinistra1 = true;
	else if( stato_attuale_1 == DESTRA ) destra1 = true;
	else if( stato_attuale_1 == GIU ) giu1 = true;
	else if( stato_attuale_1 == SU ) su1 = true;
	else if( stato_attuale_1 == FERMO ) sinistra1 = destra1 = giu1 = su1 = false;

	if( stato_attuale_2 == SINISTRA ) sinistra2 = true;
	else if( stato_attuale_2 == DESTRA ) destra2 = true;
	else if( stato_attuale_2 == GIU ) giu2 = true;
	else if( stato_attuale_2 == SU) su2 = true;
	else if( stato_attuale_2 == FERMO ) sinistra2 = destra2 = giu2 = su2 = false;


	int direzione_1 = 1;
	int direzione_2 = 1;

	direzione_1 = getch();

	switch( direzione_1 )
	{
		case KEY_LEFT :
			if( ( mov_precedente1 == 6 || destra1 == true ) && serpe1.get_lunghezza() > 1 )
				break;
			mov_precedente1 = 0;

			stato_attuale_1 = SINISTRA;
			sinistra1 = true;
			giu1 = false, destra1 = false, su1 = false;
			break;

		case KEY_RIGHT :
			if( ( mov_precedente1 == 4 || sinistra1 == true ) && serpe1.get_lunghezza() > 1 )
				break;
			mov_precedente1 = 0;

			stato_attuale_1 = DESTRA;
			destra1 = true;
			giu1 = false, sinistra1 = false, su1 = false;
			break;

		case KEY_UP :
			if( ( mov_precedente1 == 2 || giu1 == true ) && serpe1.get_lunghezza() > 1 )
				break;
			mov_precedente1 = 0;

			stato_attuale_1 = SU;
			su1 = true;
			giu1 = false, sinistra1 = false, destra1 = false;
			break;

		case KEY_DOWN :
			if( ( mov_precedente1 == 8 || su1 == true ) && serpe1.get_lunghezza() > 1 )
				break;
			mov_precedente1 = 0;

			stato_attuale_1 = GIU;
			giu1 = true;
			su1 = false, sinistra1 = false, destra1 = false;
			break;
			
		case 'a' :
			if( ( mov_precedente2 == 6 || destra2 == true ) && serpe2.get_lunghezza() > 1 )
				break;
			mov_precedente2 = 0;
			
			stato_attuale_2 = SINISTRA;
			sinistra2 = true;
			giu2 = false, destra2 = false, su2 = false;
			break;

		case 'd' :
			if( ( mov_precedente2 == 4 || sinistra2 == true ) && serpe2.get_lunghezza() > 1 )
				break;
			mov_precedente2 = 0;
			
			stato_attuale_2 = DESTRA;
			destra2 = true;
			giu2 = false, sinistra2 = false, su2 = false;
			break;

		case 'w' :
			if( ( mov_precedente2 == 2 || giu2 == true ) && serpe2.get_lunghezza() > 1 )
				break;
			mov_precedente2 = 0;
			
			stato_attuale_2 = SU;
			su2 = true;
			giu2 = false, sinistra2 = false, destra2 = false;
			break;

		case 's' :
			if( ( mov_precedente2 == 8 || su2 == true ) && serpe2.get_lunghezza() > 1 )
				break;
			mov_precedente2 = 0;
			
			stato_attuale_2 = GIU;
			giu2 = true;
			su2 = false, sinistra2 = false, destra2 = false;
			break;

		case 'p' :
			stato_gioco = PAUSA;

			if( su1 ) mov_precedente1 = 8;
			else if( sinistra1 ) mov_precedente1 = 4;
			else if( destra1 ) mov_precedente1 = 6;
			else if( giu1 ) mov_precedente1 = 2;

			if( su2 ) mov_precedente2 = 8;
			else if( sinistra2 ) mov_precedente2 = 4;
			else if( destra2 ) mov_precedente2 = 6;
			else if( giu2 ) mov_precedente2 = 2;

			sinistra1 = destra1 = giu1 = su1 = false;
			sinistra2 = destra2 = giu2 = su2 = false;
			break;

		case 27 :
			stato_gioco = ESCI;
			break;
	}

	direzione_2 = getch();


	if( ( direzione_1 == KEY_UP || direzione_1 == KEY_DOWN || direzione_1 == KEY_RIGHT || direzione_1 == KEY_LEFT ) && 
	    ( direzione_2 == KEY_UP || direzione_2 == KEY_DOWN || direzione_2 == KEY_RIGHT || direzione_2 == KEY_LEFT ) )
	    return true;
	    
	if( ( direzione_1 == 'a' || direzione_1 == 's' || direzione_1 == 'd' || direzione_1 == 'w' ) && 
	    ( direzione_2 == 'a' || direzione_2 == 's' || direzione_2 == 'd' || direzione_2 == 'w' ) )
	    return true;
	    
	switch( direzione_2 )
	{
		case KEY_LEFT :
			if( ( mov_precedente1 == 6 || destra1 == true ) && serpe1.get_lunghezza() > 1 )
				break;
			mov_precedente1 = 0;

			stato_attuale_1 = SINISTRA;
			sinistra1 = true;
			giu1 = false, destra1 = false, su1 = false;
			break;

		case KEY_RIGHT :
			if( ( mov_precedente1 == 4 || sinistra1 == true ) && serpe1.get_lunghezza() > 1 )
				break;
			mov_precedente1 = 0;

			stato_attuale_1 = DESTRA;
			destra1 = true;
			giu1 = false, sinistra1 = false, su1 = false;
			break;

		case KEY_UP :
			if( ( mov_precedente1 == 2 || giu1 == true ) && serpe1.get_lunghezza() > 1 )
				break;
			mov_precedente1 = 0;

			stato_attuale_1 = SU;
			su1 = true;
			giu1 = false, sinistra1 = false, destra1 = false;
			break;

		case KEY_DOWN :
			if( ( mov_precedente1 == 8 || su1 == true ) && serpe1.get_lunghezza() > 1 )
				break;
			mov_precedente1 = 0;

			stato_attuale_1 = GIU;
			giu1 = true;
			su1 = false, sinistra1 = false, destra1 = false;
			break;
			
		case 'a' :
			if( ( mov_precedente2 == 6 || destra2 == true ) && serpe2.get_lunghezza() > 1 )
				break;
			mov_precedente2 = 0;
			
			stato_attuale_2 = SINISTRA;
			sinistra2 = true;
			giu2 = false, destra2 = false, su2 = false;
			break;

		case 'd' :
			if( ( mov_precedente2 == 4 || sinistra2 == true ) && serpe2.get_lunghezza() > 1 )
				break;
			mov_precedente2 = 0;
			
			stato_attuale_2 = DESTRA;
			destra2 = true;
			giu2 = false, sinistra2 = false, su2 = false;
			break;

		case 'w' :
			if( ( mov_precedente2 == 2 || giu2 == true ) && serpe2.get_lunghezza() > 1 )
				break;
			mov_precedente2 = 0;
			
			stato_attuale_2 = SU;
			su2 = true;
			giu2 = false, sinistra2 = false, destra2 = false;
			break;

		case 's' :
			if( ( mov_precedente2 == 8 || su2 == true ) && serpe2.get_lunghezza() > 1 )
				break;
			mov_precedente2 = 0;
			
			stato_attuale_2 = GIU;
			giu2 = true;
			su2 = false, sinistra2 = false, destra2 = false;
			break;

		case 'p' :
			stato_gioco = PAUSA;

			if( su1 ) mov_precedente1 = 8;
			else if( sinistra1 ) mov_precedente1 = 4;
			else if( destra1 ) mov_precedente1 = 6;
			else if( giu1 ) mov_precedente1 = 2;

			if( su2 ) mov_precedente2 = 8;
			else if( sinistra2 ) mov_precedente2 = 4;
			else if( destra2 ) mov_precedente2 = 6;
			else if( giu2 ) mov_precedente2 = 2;

			sinistra1 = destra1 = giu1 = su1 = false;
			sinistra2 = destra2 = giu2 = su2 = false;
			break;

		case 27 :
			stato_gioco = ESCI;
			break;
	}    

	if( su1 )
		serpe1.muovi_su( area );
	else if( giu1 )
		serpe1.muovi_giu( area );
	else if( destra1 )
		serpe1.muovi_destra( area );
	else if( sinistra1 )
		serpe1.muovi_sinistra( area );

	if( su2 )
		serpe2.muovi_su( area );
	else if( giu2 )
		serpe2.muovi_giu( area );
	else if( destra2 )
		serpe2.muovi_destra( area );
	else if( sinistra2 )
		serpe2.muovi_sinistra( area );
	
	return false;
}
